-- interesting stuff in CommodityStats:
	-- CommodityStats.statistics: contains all scanned data in a 2-dimensional array with the format statistics[itemID][timestamp]
	-- CommodityStats.plugins: contains all helper methods to add search plugins. See examples below.
-- Recommended width of a listitem: 580px
-- Dimensions of the options space: width: 210px, height: 350px

require "Window"
 
local CommodityStatsPluginExamples = {} 
local commodityStats
 
function CommodityStatsPluginExamples:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    return o
end

function CommodityStatsPluginExamples:Init()
	-- Mark CommodityStats as a dependency
    Apollo.RegisterAddon(self, false, "", {"CommodityStats"})
end
 
function CommodityStatsPluginExamples:OnLoad()
	CommodityStatsPluginExamples.xml = XmlDoc.CreateFromFile("CommodityStatsPluginExamples.xml")
	commodityStats = Apollo.GetAddon("CommodityStats")

	-- Supported arguments for AddSearch:
		-- buttonText = name of the entry in the search menu
		-- searchCallback: Code that gets executed when your search is selected. Can be nil. Useful for when you don't have any search options or for setting initial parameters
		-- searchWindowXML: Xml document containing the search option window. Can be nil if there is no option window
		-- searchWindowName: name of the search option Window. Can be nil if there is no option window
		-- tHandler: eventhandler for the search option window, usually self.
	commodityStats.plugins:AddSearch("'Buy now' to Vendor", GetVendorables, nil, nil, self)
	commodityStats.plugins:AddSearch("'Buy Order' to Vendor", GetBuyOrderVendorables, nil, nil, self)
	commodityStats.plugins:AddSearch("Buy Order VS Sell Order", GetBuyVSSell, nil, nil, self)
	commodityStats.plugins:AddSearch("Top 1 VS Top 10 Sell price", nil, CommodityStatsPluginExamples.xml, "OptionWindow", self)
end

function GetVendorables()
	local cs = Apollo.GetAddon("CommodityStats")
	local results = {}
	for itemid, stats in pairs(cs.statistics) do
		local lastTime = table.maxn(stats)
		if lastTime > 0 then
			if stats[lastTime].sellOrderCount > 0 then
				local item = Item.GetDataFromId(itemid)
				local profit = 0
				if item ~= nil then
					local vendorPrice = item:GetSellPrice()
					if vendorPrice ~= nil then
						local sellprice = stats[lastTime].sellPrices.top1
						profit = vendorPrice:GetAmount() - sellprice
					end
				end
				if profit > 0 then
					-- AddTextSearchResult accepts a string that will be shown as a simple list item and an optional height of the listitem in pixels.
					-- The listitem supports XML textformatting. For basic documentation on text markup: http://wiki.wildstarnasa.com/index.php?title=Text_Markup
					table.insert(results, {name = item:GetName(), price = profit})
				end
			end
		end
	end
	table.sort(results, Sort)
	for i, result in ipairs(results) do
		cs.plugins:AddTextSearchResult("<T Font='CRB_Button'>" .. result.name .. ".      </T><T>Profit: </T><T TextColor='ff44ff44'>" .. FormatMoney(result.price) .. "</T>")
	end
end

function GetBuyOrderVendorables()
	local cs = Apollo.GetAddon("CommodityStats")
	local results = {}
	for itemid, stats in pairs(cs.statistics) do
		local lastTime = table.maxn(stats)
		if lastTime > 0 then
			if stats[lastTime].buyOrderCount > 0 and stats[lastTime].sellOrderCount > 0 then
				local item = Item.GetDataFromId(itemid)
				local profit = 0
				if item ~= nil then
					local vendorPrice = item:GetSellPrice()
					if vendorPrice ~= nil then
						local buyPrice = stats[lastTime].buyPrices.top1
						profit = vendorPrice:GetAmount() - buyPrice
					end
				end
				if profit > 0 then
					-- AddTextSearchResult accepts a string that will be shown as a simple list item and an optional height of the listitem in pixels.
					-- The listitem supports XML textformatting. For basic documentation on text markup: http://wiki.wildstarnasa.com/index.php?title=Text_Markup
					table.insert(results, {name = item:GetName(), price = profit})
				end
			end
		end
	end
	table.sort(results, Sort)
	for i, result in ipairs(results) do
		cs.plugins:AddTextSearchResult("<T Font='CRB_Button'>" .. result.name .. ".      </T><T>Profit: </T><T TextColor='ff44ff44'>" .. FormatMoney(result.price) .. "</T>")
	end
end

function GetBuyVSSell()
	local cs = Apollo.GetAddon("CommodityStats")
	local results = {}
	for itemid, stats in pairs(cs.statistics) do
		local item = Item.GetDataFromId(itemid)
		if item ~= nil then
			local lastTime = table.maxn(stats)
			if lastTime > 0 then
				if stats[lastTime].sellOrderCount > 0 and stats[lastTime].buyOrderCount > 0 then
					local priceDiff = stats[lastTime].sellPrices.top1 - stats[lastTime].buyPrices.top1
					if priceDiff >= 10000 then
						table.insert(results, {name = item:GetName(), price = priceDiff})
					end
				end
			end
		end
	end
	table.sort(results, Sort)
	for i, result in ipairs(results) do
		cs.plugins:AddTextSearchResult("<T Font='CRB_Button'>" .. result.name .. ".      </T><T>Profit: </T><T TextColor='ff44ff44'>" .. FormatMoney(result.price) .. "</T>")
	end
end

function GetTop1VSTop10Sell(minimumDiff, maxSelltop1)
	local cs = Apollo.GetAddon("CommodityStats")
	-- Results get cleared automatically when you switch search plugins. For other cases, use ClearSearchResults
	cs.plugins:ClearSearchResults()
	local resultCounter = 0
	for itemid, stats in pairs(cs.statistics) do
		local lastTime = table.maxn(stats)
		if lastTime > 0 then
			if stats[lastTime].sellOrderCount > 0 then
				local item = Item.GetDataFromId(itemid)
				if item ~= nil then
					local diff = stats[lastTime].sellPrices.top10 - stats[lastTime].sellPrices.top1
					if diff >= minimumDiff and stats[lastTime].sellPrices.top1 <= maxSelltop1 then -- is the difference between the top1 and top10 sell price more than 10 gold?

						-- AddListItemSearchResult is for sending custom listitems and returns the created item. It accepts the following arguments:
							-- xml: the xmldoc containing the window
							-- windowName: the name of the window to send
							-- tHandler: eventhandler for the window, usually self
						local listItem = cs.plugins:AddListItemSearchResult(CommodityStatsPluginExamples.xml, "ResultListItem", self)
						listItem:FindChild("txtItemName"):SetText(item:GetName())
						listItem:FindChild("monTop1"):SetAmount(stats[lastTime].sellPrices.top1)
						listItem:FindChild("monTop10"):SetAmount(stats[lastTime].sellPrices.top10)
						listItem:FindChild("monDifference"):SetAmount(diff)
						resultCounter = resultCounter + 1
					end
				end
			end
		end
	end
	if resultCounter == 0 then
		cs.plugins:AddTextSearchResult("No results match this search criteria.")
	end
end


function CommodityStatsPluginExamples:OnSearchTop1vTop10( wndHandler, wndControl, eMouseButton )
	local wndOptions = wndControl:GetParent()
	local minimumDiff = wndOptions:FindChild("monMinimumDifference"):GetAmount()
	local maxSelltop1 = wndOptions:FindChild("monMaxTop1Price"):GetAmount()

	GetTop1VSTop10Sell(minimumDiff, maxSelltop1)
end

function FormatMoney(nAmount)
    -- careful with negative amounts when calculating modulo
    local isNegative = nAmount < 0
    if isNegative then nAmount = nAmount * -1 end

    local platinum = math.floor(nAmount / 1000000)
    nAmount = nAmount % 1000000
    local gold = math.floor(nAmount / 10000)
    nAmount = nAmount % 10000
    local silver = math.floor(nAmount / 100)
    local copper = nAmount % 100
    local output = ""
    if platinum > 0 then output = output .. tostring(platinum) .. "p " end
    if gold > 0 then output = output .. tostring(gold) .. "g " end
    if silver > 0 then output = output .. tostring(silver) .. "s " end
    output = output .. tostring(math.floor(copper)) .. "c"
    if isNegative then output = "-" .. output end
    return output
end

function Sort(objA, objB)
	return objA.price > objB.price
end

local CommodityStatsPluginExamplesInst = CommodityStatsPluginExamples:new()


CommodityStatsPluginExamplesInst:Init()

